#include <stm32f10x.h>
#include <stm32f10x_rcc.h>
#include <stm32f10x_tim.h>
#include <misc.h>
#include "i2c_dma.h"
#include "at24c04b.h"

#define PAGESIZE 16
//   ,       
//      .
#define MAX_NO_WRITES(x) (PAGESIZE - (x % PAGESIZE))

static uint8_t inProcess; // 

static void at24c04b_initTimer(void);

void at24c04b_init(uint8_t whichI2C){

 if ( whichI2C == 1 ){
	 i2c_lowLevel_init(I2C1, I2CwithDMA);
 }else if ( whichI2C == 2 ){
	 i2c_lowLevel_init(I2C2, I2CwithDMA);
 } else if ( whichI2C == 3 ){
	 i2c_lowLevel_init(I2C1, I2CwithDMA);
	 i2c_lowLevel_init(I2C2, I2CwithDMA);
 }

  at24c04b_initTimer();

}

void at24c04b_Write(struct at24c04b_deviceInfo * devInf, uint8_t * buf, uint32_t nbyte, uint16_t offset){

	uint16_t factn, tail;
	uint16_t maxn = MAX_NO_WRITES(offset);
	uint8_t backup; // ?

	if ( offset > 511 )
		return;

	factn = ( maxn > nbyte) ?  nbyte: maxn;

	if ( I2CwithDMA ){
		//   ( 9)      
		*(buf-1) = (uint8_t)offset;

		//         
		if ( offset / 16 < 16){ // offset >= 256 ?     .
			i2c_dma_write(devInf->I2Cx, buf-1, factn+1, devInf->devAddress | DEVaddressConst);
		} else {
			i2c_dma_write(devInf->I2Cx, buf-1, factn+1, devInf->devAddress | DEVaddressConst | fifthPageBit);
		}
	}else{

		//         
		if ( offset / 16 < 16){
			i2c_write(devInf->I2Cx, buf, factn, devInf->devAddress | DEVaddressConst, offset);
		} else {
			i2c_write(devInf->I2Cx, buf, factn, devInf->devAddress | DEVaddressConst | fifthPageBit, offset);
		}
	}

	//    
	while ( ! at24c04b_Test(devInf) )
		;



	offset += factn;
	tail = nbyte - factn;

	while ( tail > 0 ){

		if ( offset > 511 )
			return;

	//     
		factn = ( tail > PAGESIZE) ?  PAGESIZE: tail;

		if ( I2CwithDMA ){

			backup = *(buf+maxn-1);//    
			*(buf+maxn-1) = (uint8_t)offset;

			if ( offset / 16 < 16){
				i2c_dma_write(devInf->I2Cx, buf+maxn-1, factn+1, devInf->devAddress | DEVaddressConst);
			} else {
				i2c_dma_write(devInf->I2Cx, buf+maxn-1, factn+1, devInf->devAddress | DEVaddressConst | fifthPageBit);
			}

			*(buf+maxn-1) = backup;//    

		}else{
			if ( offset / 16 < 16){
				i2c_write(devInf->I2Cx, buf+maxn, factn, devInf->devAddress | DEVaddressConst, offset);
			} else {
				i2c_write(devInf->I2Cx, buf+maxn, factn, devInf->devAddress | DEVaddressConst | fifthPageBit, offset);
			}
		}

		//    
		while ( ! at24c04b_Test(devInf) )
			;

			offset += factn;
			maxn += factn;
			tail -= factn;
	}



}

void at24c04b_Read(struct at24c04b_deviceInfo * devInf, uint8_t * buf, uint32_t nbyte, uint16_t offset){

	if ( offset > 511 )
		return;

	if ( offset / 16 < 16){
		if ( I2CwithDMA ){
			i2c_dma_read(devInf->I2Cx, buf, nbyte, devInf->devAddress | DEVaddressConst, offset);
		}else
			i2c_read(devInf->I2Cx, buf, nbyte, devInf->devAddress | DEVaddressConst, offset, 0);

	} else {
		if ( I2CwithDMA ){
			i2c_dma_read(devInf->I2Cx, buf, nbyte, devInf->devAddress | DEVaddressConst | fifthPageBit, offset);
		}else
			i2c_read(devInf->I2Cx, buf, nbyte, devInf->devAddress | DEVaddressConst | fifthPageBit, offset, 0);
	}



}

static void at24c04b_initTimer(void){

	TIM_TimeBaseInitTypeDef TIM_TimeBaseStructure;
	NVIC_InitTypeDef NVIC_InitStructure;

	RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM2, ENABLE);

	NVIC_InitStructure.NVIC_IRQChannel = TIM2_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 3;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelCmd = DISABLE;
	NVIC_Init(&NVIC_InitStructure);

	TIM_TimeBaseStructInit(&TIM_TimeBaseStructure );
	TIM_TimeBaseStructure.TIM_Prescaler = SystemCoreClock /2000 - 1; // "12000" ( 0.5 ms per tick )
	TIM_TimeBaseStructure.TIM_Period = 2 - 1; // 1 
	TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
	TIM_TimeBaseInit (TIM2, &TIM_TimeBaseStructure );

	TIM_ITConfig(TIM2, TIM_IT_Update, ENABLE); //  
}

void TIM2_IRQHandler(void){

	if ( TIM2->SR & TIM_FLAG_Update ){

		TIM2->SR = (uint16_t)~TIM_IT_Update;
		//asm("nop");
		//asm("nop");

		NVIC_DisableIRQ(TIM2_IRQn);
		TIM2->CR1 &= (uint16_t)(~((uint16_t)TIM_CR1_CEN)); //TIM_Cmd(TIM2, DISABLE);
		inProcess = 0;

	}
}

void at24c04b_CurrentAddressRead(struct at24c04b_deviceInfo * devInf, uint8_t * buf){

	i2c_read(devInf->I2Cx, buf, 0, devInf->devAddress | DEVaddressConst, 0, 1);

}

uint8_t at24c04b_Test(struct at24c04b_deviceInfo * devInf){

	// 

	TIM2->SR = (uint16_t)~TIM_IT_Update;
	NVIC_EnableIRQ(TIM2_IRQn);

	inProcess = 1;
	//   URS  ,   TIM_FLAG_Update 
	//   
	TIM2->CR1 |= TIM_CR1_CEN  | TIM_CR1_URS;     // ENABLE tim1

	//
	while ( inProcess )
		;

	return  i2c_test(devInf->I2Cx, devInf->devAddress | DEVaddressConst );

}
