
#ifndef AT24C04B_H_
#define AT24C04B_H_

// -----------------------------------------------------------

//    at24c04b_Write()      
//    .
// !     DMA:
// 1.           .
//        .
// :
// uint8_t dataBuff[20];
// uint8_t * userData = dataBuff + 1;
// userData   .
//     at24c04b_Read() .
// 2.  :
//     (   - )   , 
//          (   
// ,   ).    
//      .       .  
//     (  )  
//     ..         
//   .    .      .  
//      -   .    
//   .
#define I2CwithDMA (uint8_t) 1   //  0 ( DMA)  1 ( DMA)

//  ---------------------------------------------------

#include <stm32f10x.h>

//    
struct at24c04b_deviceInfo{
	I2C_TypeDef* I2Cx;
	uint8_t devAddress; //    devN1...devN4
};

//   "1010 AA P R/W"
//     AA ( devN1...devN4)

//   eeprom at24c04b   
#define devN1 (uint8_t) 0x0; //xxxx00xx
#define devN2 (uint8_t) 0x4; //xxxx01xx
#define devN3 (uint8_t) 0x8; //xxxx10xx
#define devN4 (uint8_t) 0x12; //xxxx11xx

//      
// ( 9  )
#define DEVaddressConst 0xA0

// ( )   
//   ( 0)   
#define fifthPageBit 2

//   I2C.
// whichI2C:
//  1 -    I2C1
//  2 - I2C2
//  3 -   .
//   (    )
void at24c04b_init(uint8_t whichI2C);

//     EEPROM
//       EEPROM    .
//devInf -   
//buf -     (  ,  -  )
//nbyte -   
//offset - (),     (  0  511)
// (,   512    .)
void at24c04b_Write(struct at24c04b_deviceInfo * devInf, uint8_t * buf, uint32_t nbyte, uint16_t offset);

//   EEPROM  .
//
//devInf -   
//buf -   
//nbyte -   
//offset - (),     (  0  511)
//
// (          (511 ),     0
//   .
// ,   4    510.   2  (  ) 
//  2    0.)
void at24c04b_Read(struct at24c04b_deviceInfo * devInf, uint8_t * buf, uint32_t nbyte, uint16_t offset);

//  (1 )  ,     
//  ()
// : 1 -  , 0 -   .
uint8_t at24c04b_Test(struct at24c04b_deviceInfo * devInf);

//  .
//     (      ; 
// "" ,        + 1)
//    (9 )  ...
//   .
//
//  EEPROM    "255". ,  .
// (    ,      )
//       .
void at24c04b_CurrentAddressRead(struct at24c04b_deviceInfo * devInf, uint8_t * buf);

#endif
