#include <stm32f10x.h>
#include <stm32f10x_gpio.h>
#include <stm32f10x_rcc.h>
#include <stm32f10x_dma.h>
#include "i2c_dma.h"

#define RW_toRead (uint8_t)1

static uint8_t emptyVal; //    DMA

// .  
static void i2c_dma_Init(void);

//    
static void dmaTrnsmtChannel_Init(I2C_TypeDef* I2Cx, const void *tbuf , unsigned count);

//    
static void dmaRcvChannel_Init(I2C_TypeDef* I2Cx, void *rbuf , unsigned count);

//     "" .    I2C_SR1_ADDR.


void i2c_lowLevel_init(I2C_TypeDef* I2Cx, uint8_t withDMA){

	GPIO_InitTypeDef GPIO_InitStruct;

	//   B   
	RCC->APB2ENR |= (RCC_APB2Periph_GPIOB | RCC_APB2Periph_AFIO);

	GPIO_InitStruct.GPIO_Mode = GPIO_Mode_AF_OD;
	GPIO_InitStruct.GPIO_Speed = GPIO_Speed_10MHz;

	if ( I2Cx == I2C1 ){

		RCC->APB1ENR |= RCC_APB1Periph_I2C1; // I2C1

		GPIO_InitStruct.GPIO_Pin = GPIO_Pin_6 | GPIO_Pin_7; // SCL , SDA
		GPIO_Init(GPIOB, &GPIO_InitStruct);

	}else if ( I2Cx == I2C2 ){

		RCC->APB1ENR |= RCC_APB1Periph_I2C2; // I2C1

		GPIO_InitStruct.GPIO_Pin = GPIO_Pin_10 | GPIO_Pin_11;
		GPIO_Init(GPIOB, &GPIO_InitStruct);

	}

	//  :
	// T_high(   .  ) = 0.7 
	// T_Low(   .   ) = 1.775 
	//  TRISE:  300  / ( 1/24 ) + 1;  300      EEPROM (Inputs Rise Time)
	//     (SCL)  400 
	//
	// ( CCR = 13 ( Tlow/Thigh = 2),   = 615 , T_high = 0.45 , T_Low = 1.175 )
	// ( FREQ APB1 = 24 MHz    Tlow/Thigh = 16/9  )

	I2Cx->CR2 &= ~I2C_CR2_FREQ;
	I2Cx->CR2 |= 24; // FREQ APB1 = 24 MHz
	I2Cx->CCR = 0x8014;  // FastMode, Tlow/Thigh = 2, CCR = 20;
	I2Cx->TRISE = 8+1; //   eeprom (tR) Inputs Rise Time = 0.3 mcs

	if ( withDMA ){
		i2c_dma_Init();
		dmaTrnsmtChannel_Init(I2Cx, &emptyVal, 1); // 
		dmaRcvChannel_Init(I2Cx, &emptyVal, 1); // 
	}

}

static void i2c_dma_Init(void){

		RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);

}

void i2c_write(I2C_TypeDef* I2Cx, const uint8_t * buf, uint32_t nbyte, uint8_t SlaveAddress, uint16_t offset){

	uint32_t i;
	uint16_t dummyRead;

	I2Cx->CR1 |= I2C_CR1_PE; // enable
	I2Cx->CR1 |= I2C_CR1_START; // to send Start condition

	while ( !(I2Cx->SR1 & I2C_SR1_SB) )
		;

	I2Cx->DR = SlaveAddress;

	while ( !(I2Cx->SR1 & I2C_SR1_ADDR) )
		;
	dummyRead = I2Cx->SR2; //   ADDR  SR1

	// ,     
	I2Cx->DR = offset;

	while ( !(I2Cx->SR1 & I2C_SR1_TXE) )
		;
	// .  

	for (i = 0; i < nbyte; i++){
		//  (),   
		I2Cx->DR = buf[i];

		while ( !(I2Cx->SR1 & I2C_SR1_TXE) )
			;
		//BTF
		// .  
	}



	while ( !(I2Cx->SR1 & I2C_SR1_BTF) )
		;

	I2Cx->CR1 |= I2C_CR1_STOP; // to send Stop condition

	I2Cx->CR1 &= ~I2C_CR1_PE; // disable


}

void i2c_dma_write(I2C_TypeDef* I2Cx, const uint8_t * buf, uint32_t nbyte, uint8_t SlaveAddress){

	DMA_Channel_TypeDef *txChan;
	uint32_t DMA1_FLAG_TC;
	uint16_t dummyRead;

	I2Cx->CR1 |= I2C_CR1_PE; // enable
	I2Cx->CR1 |= I2C_CR1_START; // to send Start condition

	while ( !(I2Cx->SR1 & I2C_SR1_SB) )
		;

	I2Cx->DR = SlaveAddress;

		//DMA---------------------
		if ( I2Cx == I2C1 ){
			txChan = DMA1_Channel6;
			DMA1_FLAG_TC = DMA1_FLAG_TC6;
		} else if ( I2Cx == I2C2 ){
			txChan = DMA1_Channel4;
			DMA1_FLAG_TC = DMA1_FLAG_TC4;
		}

		txChan->CPAR = (uint32_t)(&(I2Cx->DR));
		txChan->CNDTR = nbyte;
		txChan->CMAR = (uint32_t)buf;
		txChan->CCR |= DMA_CCR1_EN;
		I2Cx->CR2 |= I2C_CR2_DMAEN;

		//I2C---------------------
		while ( !(I2Cx->SR1 & I2C_SR1_ADDR) )
			;
		dummyRead = I2Cx->SR2; //   ADDR  SR1

		//DMA---------------------
		// Wait for completion
		while (DMA_GetFlagStatus(DMA1_FLAG_TC) == RESET);//   
		DMA1->IFCR = DMA1_FLAG_TC; // clear

		// Disable channel
		txChan->CCR &= (uint16_t)(~DMA_CCR1_EN); //DMA_Cmd(txChan, DISABLE);
		// DMA Requests Disable
		I2Cx->CR2 &= ~I2C_CR2_DMAEN;


		//I2C---------------------
	while ( !(I2Cx->SR1 & I2C_SR1_BTF) )
		;

	I2Cx->CR1 |= I2C_CR1_STOP; // to send Stop condition

	I2Cx->CR1 &= ~I2C_CR1_PE; // disable


}

//  SlaveAddress      RW  1.     .
// (        ( "").)
void i2c_read(I2C_TypeDef* I2Cx, uint8_t * buf, uint32_t nbyte, uint8_t SlaveAddress, uint16_t offset, uint8_t CAR){

	uint16_t dummyRead;
	uint32_t i;

	I2Cx->CR1 |= I2C_CR1_PE; // enable

	if ( !CAR ) {
		I2Cx->CR1 |= I2C_CR1_START; // to send Start condition

		while ( !(I2Cx->SR1 & I2C_SR1_SB) )
			;
		// ""         eeprom

		I2Cx->DR = SlaveAddress; // 0 - 

		while ( !(I2Cx->SR1 & I2C_SR1_ADDR) )
			;
		dummyRead = I2Cx->SR2; //   ADDR  SR1

		//  " "
		I2Cx->DR = offset;

		while ( !(I2Cx->SR1 & I2C_SR1_TXE) )
			;
		// .  

	} else {
		nbyte = 1;
	}

	//  Start condition     
	I2Cx->CR1 |= I2C_CR1_START; // to send Start condition
	while ( !(I2Cx->SR1 & I2C_SR1_SB) )
		;

	I2Cx->DR = SlaveAddress | RW_toRead; // 0 - 

	//        ,       

	if ( nbyte > 2 ){

		I2Cx->CR1 |= I2C_CR1_ACK;

		while ( !(I2Cx->SR1 & I2C_SR1_ADDR) )
			;
		dummyRead = I2Cx->SR2; //   ADDR  SR1

		for( i = 0; i < nbyte-3; i++ ){

			while ( !(I2Cx->SR1 & I2C_SR1_RXNE) ) //  
				;

			buf[i] = (uint8_t)I2Cx->DR;

		}

		//    
		while ( !(I2Cx->SR1 & I2C_SR1_RXNE) ) //  
			;
		while ( !(I2Cx->SR1 & I2C_SR1_BTF) ) //   BTF
			;

		I2Cx->CR1 &= ~I2C_CR1_ACK; // 
		buf[i++] = (uint8_t)I2Cx->DR; // N-2

		I2Cx->CR1 |= I2C_CR1_STOP; // to send Stop condition
		buf[i++] = (uint8_t)I2Cx->DR; // N-1
		while ( !(I2Cx->SR1 & I2C_SR1_RXNE) ) //  
			;
		buf[i] = (uint8_t)I2Cx->DR; // N

	}else if ( nbyte == 2 ){

		I2Cx->CR1 |= I2C_CR1_POS;
		I2Cx->CR1 |= I2C_CR1_ACK;

		while ( !(I2Cx->SR1 & I2C_SR1_ADDR) )
			;
		dummyRead = I2Cx->SR2; //   ADDR  SR1
		I2Cx->CR1 &= ~I2C_CR1_ACK; // 
		while ( !(I2Cx->SR1 & I2C_SR1_BTF) ) //   BTF
			;

		I2Cx->CR1 |= I2C_CR1_STOP; // to send Stop condition

		buf[0] = (uint8_t)I2Cx->DR;
		buf[1] = (uint8_t)I2Cx->DR;

	}else if ( nbyte == 1 ){


		while ( !(I2Cx->SR1 & I2C_SR1_ADDR) )
			;


		I2Cx->CR1 &= ~I2C_CR1_ACK; // 
		dummyRead = I2Cx->SR2; //   ADDR  SR1

		I2Cx->CR1 |= I2C_CR1_STOP; // to send Stop condition

		while ( !(I2Cx->SR1 & I2C_SR1_RXNE) ) //  
			;

		buf[0] = (uint8_t)I2Cx->DR;

	}



	I2Cx->CR1 &= ~I2C_CR1_PE; // disable
}


uint8_t i2c_test(I2C_TypeDef* I2Cx, uint8_t SlaveAddress){

	uint16_t dummyRead, res;
	uint32_t i;

	I2Cx->CR1 |= I2C_CR1_PE; // enable
	I2Cx->CR1 |= I2C_CR1_START; // to send Start condition

	while ( !(I2Cx->SR1 & I2C_SR1_SB) )
		;

	I2Cx->DR = SlaveAddress; // 0 - 

	//  20    ACK
	for ( i = 0; i < 50; i++ ){  // !!!
		res = I2Cx->SR1 & I2C_SR1_ADDR;
		if ( res != 0)
			break;
	}

	dummyRead = I2Cx->SR2; //   ADDR  SR1

	I2Cx->CR1 |= I2C_CR1_STOP; // to send Stop condition

	I2Cx->CR1 &= ~I2C_CR1_PE; // disable

	if ( res == 0 )
		return (uint8_t)0;
	else
		return (uint8_t)1;

}

void i2c_dma_read(I2C_TypeDef* I2Cx, uint8_t * buf, uint32_t nbyte, uint8_t SlaveAddress, uint16_t offset){

	DMA_Channel_TypeDef *rxChan;
	uint32_t DMA1_FLAG_TC;
	uint16_t dummyRead;

	I2Cx->CR1 |= I2C_CR1_PE; // enable

	I2Cx->CR1 |= I2C_CR1_START; // to send Start condition

	while ( !(I2Cx->SR1 & I2C_SR1_SB) )
		;
	// ""         eeprom

	I2Cx->DR = SlaveAddress; // 0 - 

	while ( !(I2Cx->SR1 & I2C_SR1_ADDR) )
		;
	dummyRead = I2Cx->SR2; //   ADDR  SR1

	//  " "
	I2Cx->DR = offset;

	while ( !(I2Cx->SR1 & I2C_SR1_TXE) )
		;
	// .  


	//  Start condition     
	I2Cx->CR1 |= I2C_CR1_START; // to send Start condition
	while ( !(I2Cx->SR1 & I2C_SR1_SB) )
		;

	I2Cx->DR = SlaveAddress | RW_toRead; // 0 - 



	//            

	if ( nbyte >= 2 ){

		//DMA----------------------
		if ( I2Cx == I2C1 ){
			rxChan = DMA1_Channel7;
			DMA1_FLAG_TC = DMA1_FLAG_TC7;
		} else if ( I2Cx == I2C2 ){
			rxChan = DMA1_Channel5;
			DMA1_FLAG_TC = DMA1_FLAG_TC5;
		}

		rxChan->CPAR = (uint32_t)(&(I2Cx->DR));
		rxChan->CNDTR = nbyte;
		rxChan->CMAR = (uint32_t)buf;
		rxChan->CCR |= DMA_CCR1_EN;
		I2Cx->CR2 |= I2C_CR2_DMAEN;

		//I2C---------------------

		I2Cx->CR1 |= I2C_CR1_ACK;
		I2Cx->CR2 |= I2C_CR2_LAST;

		while ( !(I2Cx->SR1 & I2C_SR1_ADDR) )
			;
		dummyRead = I2Cx->SR2; //   ADDR  SR1

		//DMA----------------------
		// Wait for completion
		while (DMA_GetFlagStatus(DMA1_FLAG_TC) == RESET);//   
		DMA1->IFCR = DMA1_FLAG_TC; // clear

		// Disable channel
		rxChan->CCR &= (uint16_t)(~DMA_CCR1_EN); //DMA_Cmd(rxChan, DISABLE);
		// DMA Requests Disable
		I2Cx->CR2 &= ~I2C_CR2_DMAEN;

		//I2C---------------------
		I2Cx->CR1 |= I2C_CR1_STOP; // to send Stop condition

		I2Cx->CR2 &= ~I2C_CR2_LAST;

	}else if ( nbyte == 1 ){ //  DMA  DMA  DMA  DMA


		while ( !(I2Cx->SR1 & I2C_SR1_ADDR) )
			;


		I2Cx->CR1 &= ~I2C_CR1_ACK; // 
		dummyRead = I2Cx->SR2; //   ADDR  SR1

		I2Cx->CR1 |= I2C_CR1_STOP; // to send Stop condition

		while ( !(I2Cx->SR1 & I2C_SR1_RXNE) ) //  
			;

		buf[0] = (uint8_t)I2Cx->DR;

	}

	I2Cx->CR1 &= ~I2C_CR1_PE; // disable
}

static void dmaRcvChannel_Init(I2C_TypeDef* I2Cx, void *rbuf , unsigned count){

DMA_InitTypeDef DMA_InitStructure;
DMA_Channel_TypeDef *rxChan;

if ( I2Cx == I2C1 ){
	rxChan = DMA1_Channel7;
	//DMA1_FLAG_TC = DMA1_FLAG_TC7;
} else if ( I2Cx == I2C2 ){
	rxChan = DMA1_Channel5;
	//DMA1_FLAG_TC = DMA1_FLAG_TC5;
}

DMA_DeInit(rxChan);

// Common to both channels
DMA_InitStructure.DMA_PeripheralBaseAddr = (uint32_t)(&(I2Cx->DR));

DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Byte;
DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_Byte;

DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
DMA_InitStructure.DMA_BufferSize = count;
DMA_InitStructure.DMA_Mode = DMA_Mode_Normal;
DMA_InitStructure.DMA_Priority = DMA_Priority_VeryHigh;
DMA_InitStructure.DMA_M2M = DMA_M2M_Disable;
// Rx Channel
DMA_InitStructure.DMA_MemoryBaseAddr = (uint32_t)rbuf;
DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;
DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralSRC;
DMA_Init(rxChan, &DMA_InitStructure);

// Tx channel is not used.

// Enable channel
//DMA_Cmd(rxChan, ENABLE);
// DMA Requests Enable
//I2Cx->CR2 |= I2C_CR2_DMAEN;

}

static void dmaTrnsmtChannel_Init(I2C_TypeDef* I2Cx, const void *tbuf , unsigned count){

DMA_InitTypeDef DMA_InitStructure;
DMA_Channel_TypeDef *txChan;
//uint32_t DMA1_FLAG_TC;

if ( I2Cx == I2C1 ){
	//rxChan = DMA1_Channel7;
	txChan = DMA1_Channel6;
	//DMA1_FLAG_TC = DMA1_FLAG_TC6;
} else if ( I2Cx == I2C2 ){
	//rxChan = DMA1_Channel5;
	txChan = DMA1_Channel4;
	//DMA1_FLAG_TC = DMA1_FLAG_TC4;
}

DMA_DeInit(txChan);

// Common to both channels
DMA_InitStructure.DMA_PeripheralBaseAddr = (uint32_t)(&(I2Cx->DR));

DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Byte;
DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_Byte;

DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
DMA_InitStructure.DMA_BufferSize = count;
DMA_InitStructure.DMA_Mode = DMA_Mode_Normal;
DMA_InitStructure.DMA_Priority = DMA_Priority_VeryHigh;
DMA_InitStructure.DMA_M2M = DMA_M2M_Disable;

// Rx Channel is not used.

// Tx channel
DMA_InitStructure.DMA_MemoryBaseAddr = (uint32_t) tbuf;
DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;
DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralDST;
DMA_Init(txChan, &DMA_InitStructure);

// Enable channel
//DMA_Cmd(txChan, ENABLE);

// DMA Requests Enable
//I2Cx->CR2 |= I2C_CR2_DMAEN;

}

