#include <stm32f10x.h>
#include <stm32f10x_rcc.h>
#include <stm32f10x_gpio.h>

#include "at24c04b.h"

#define btn1 GPIO_Pin_1
#define bufLen 33

void otherInit(void);

void Delay(uint32_t nTime);
uint8_t debounce(uint8_t last);

uint8_t dataBuff[bufLen] = {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11,
		12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32};
uint8_t * userData = dataBuff+1;

uint8_t rBuff[512];

int main(void)
{

	uint8_t currentButton;
	uint8_t lastButton;

	// 
	struct at24c04b_deviceInfo EEPROM_1;

	EEPROM_1.I2Cx = I2C1;
	EEPROM_1.devAddress = devN2;

	otherInit(); // Initialize user button

	lastButton = GPIO_ReadInputDataBit(GPIOA, btn1);
	currentButton = lastButton;

	// Configure SysTick Timer
	if (SysTick_Config(SystemCoreClock / 1000))
			while (1);

	at24c04b_init(1);

    while(1){

    	currentButton = debounce(lastButton);

		if ( lastButton == Bit_RESET && currentButton == Bit_SET ){


			at24c04b_Write(&EEPROM_1, userData, 32, 0);
			at24c04b_Write(&EEPROM_1, userData, 32, 128);
			at24c04b_Write(&EEPROM_1, userData, 32, 256);
			at24c04b_Write(&EEPROM_1, userData, 32, 384);
			at24c04b_Write(&EEPROM_1, userData, 32, 480);

			at24c04b_Read(&EEPROM_1, rBuff, 512, 0);
			//     
			//(   )
			at24c04b_Read(&EEPROM_1, rBuff, 16, 128);

		}

		lastButton = currentButton;

    }
}

void otherInit(void){
	GPIO_InitTypeDef GPIO_InitStructure;

	// user button A0
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA , ENABLE);
	GPIO_InitStructure.GPIO_Pin = btn1;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
	GPIO_Init(GPIOA,	&GPIO_InitStructure);
}

static __IO uint32_t TimingDelay;

void Delay(uint32_t nTime){
	TimingDelay = nTime;
	while(TimingDelay != 0);
}


void SysTick_Handler(void){
	if (TimingDelay != 0x00)
		TimingDelay--;
}

uint8_t debounce(uint8_t last){
	uint8_t current = GPIO_ReadInputDataBit(GPIOA, btn1);
	  if ( last != current ){
		  Delay(5);
		  current = GPIO_ReadInputDataBit(GPIOA, btn1);
	  }
  return current;
}

#ifdef USE_FULL_ASSERT
void assert_failed(uint8_t *file, uint32_t line)
{
	/* Infinite loop */
	/* Use GDB to find out why we're here */
	while (1);
}
#endif
